@@ -1,4 +1,4 @@
-import requests, datetime, time, warnings, json, calendar, math, os, re, jieba, jieba.analyse, random
+import requests, datetime, time, warnings, json, calendar, math, os, re, jieba, jieba.analyse, random, string
 from datetime import date
 from dateutil.relativedelta import relativedelta
 from pandas.io.json import json_normalize
@@ -179,6 +179,24 @@ class Qimai_Outside_Tool:
         day = self.data_info[0].weekday()  # weekday()可以获得是星期几
         return week_day[day]
 
+    def get_index_list_num(self):
+        """
+            * 定位列表内字符串位置：
+            * 返回列表；
+        """
+        run_list_num = []
+        if '' in self.data_info[0]:
+            for x, i in enumerate(self.data_info[0]):
+                if i == '':  # 判断字符串
+                    run_list_num.append(x - 1)
+            # 把最后一列加入，排版完毕后删除空值；
+            run_list_num.append(len(self.data_info[0]) - 1)
+            for x, i in enumerate(run_list_num):
+                run_list_num[x] = i - x
+        else:
+            run_list_num.append(len(self.data_info[0]) - 1)
+        return run_list_num
+
     def json_to_df(self):
         """
             * json转为pandas的表格形式；
@@ -468,7 +486,7 @@ class Qimai_Diy_Var:
     """
         * 便于其他类继承的通用参数区：
     """
-    def __init__(self, country='cn', rank_type='all', version='ios12', device='iphone', brand='all', run_time=today_date, status=6, genre_type=36, lost_sort='out_time'):
+    def __init__(self, country='cn', rank_type='all', version='ios12', device='iphone', brand='all', status=6, genre_type=36, lost_sort='out_time'):
         """
             * 类参数区：
             :param country: 国家/地区：默认中国
@@ -476,7 +494,6 @@ class Qimai_Diy_Var:
             :param version: 系统版本：默认iOS 12
             :param device: 设备类型：默认iPhone
             :param brand: 价格类型：默认全部，其他可选例如免费、付费、畅销
-            :param run_time: 获取时间，默认当日
             :param status:
             :param genre_type: 榜单类型，默认全部，其他可选输入榜单对应榜单id即可
         """
@@ -485,7 +502,6 @@ class Qimai_Diy_Var:
         self.version = version
         self.device = device
         self.brand = brand
-        self.run_time = run_time
         self.status = status
         self.genre_type = genre_type
         self.lost_sort = lost_sort
@@ -684,7 +700,7 @@ class Get_Keyword_Info(Qimai_Diy_Var):
 
     def get_keyword_search(self):
         """
-            * 获取关键词下产品信息：
+            * 获取关键词下产品信息(第一页)：
             * 默认搜索版本为iOS 12；
         """
         url = 'https://api.qimai.cn/search/index?device=%s&country=%s&search=%s&date=%s&version=%s&search_type=%s&changeRateDate=&page=1&changeRateType=top10&sdate=%s&edate=%s&status=%s' %(self.device, self.country, self.keyword, self.start_date, self.version, self.search_type, self.start_date, self.end_date, self.status)
@@ -761,6 +777,60 @@ class Get_Keyword_Info(Qimai_Diy_Var):
         self.keyword_hotSearch_data = res.json()
         return self.keyword_hotSearch_data
 
+    def get_seven_hotSearch(self):
+        """
+            * 获取关键词近7日上热搜数据：
+            * 返回第1个为次数，第2个为时长；
+        """
+        self.get_hotSearch_data()
+        hot_search_all = ['', '']
+        hot_search_data = self.keyword_hotSearch_data['data']['collect']['show_collect'][2]  # 近07日数据；
+        for i in hot_search_data:
+            title = i['title']
+            if title == '近7日上榜次数':
+                hot_search_all[0] = i['value']
+            elif title == '近7日上榜时长':
+                hot_search_all[1] = i['value']
+
+        # 返回数据；
+        return hot_search_all
+
+    def get_thirty_hotSearch(self):
+        """
+            * 获取关键词近30日上热搜数据：
+            * 返回第1个为次数，第2个为时长；
+        """
+        self.get_hotSearch_data()
+        hot_search_all = ['', '']
+        hot_search_data = self.keyword_hotSearch_data['data']['collect']['show_collect'][1]  # 近30日数据；
+        for i in hot_search_data:
+            title = i['title']
+            if title == '近30日上榜次数':
+                hot_search_all[0] = i['value']
+            elif title == '近30日上榜时长':
+                hot_search_all[1] = i['value']
+
+        # 返回数据；
+        return hot_search_all
+
+    def get_ninety_hotSearch(self):
+        """
+            * 获取关键词近90日上热搜数据：
+            * 返回第1个为次数，第2个为时长；
+        """
+        self.get_hotSearch_data()
+        hot_search_all = ['', '']
+        hot_search_data = self.keyword_hotSearch_data['data']['collect']['show_collect'][0]  # 近90日数据；
+        for i in hot_search_data:
+            title = i['title']
+            if title == '近90日上榜次数':
+                hot_search_all[0] = i['value']
+            elif title == '近90日上榜时长':
+                hot_search_all[1] = i['value']
+
+        # 返回数据；
+        return hot_search_all
+
     def get_keyword_extend(self, max_index=1000000, orderBy='', order=''):
         """
             * 关键词扩展助手接口：
@@ -831,9 +901,10 @@ class Get_App_Keyword(Qimai_Diy_Var):
         * 举例①：获取App的对应时间覆盖数据(默认当日)；
         * 举例②：获取App不同日期的T3、T5数据；
     """
-    def __init__(self, appid):
+    def __init__(self, appid, run_time=today_date):
         Qimai_Diy_Var.__init__(self)
         self.appid = appid
+        self.run_time = run_time
 
     def get_keywordDetail(self):
         """
@@ -880,7 +951,7 @@ class Get_App_Keyword(Qimai_Diy_Var):
     def get_keywordHistory_rank(self, keyword, start_date, end_date, day=1):
         """
             * 获取时间段内关键词历史排名数据；
-            * 可指定按分钟、小时、天，具体参考官网；
+            * 可指定按分钟、小时、天，具体参考官网，默认按天；
             :param keyword: 关键词
             :param start_date: 开始日期
             :param end_date: 结束日期
@@ -891,7 +962,7 @@ class Get_App_Keyword(Qimai_Diy_Var):
         self.app_keywordHistor_data = res.json()
         return self.app_keywordHistor_data
 
-    def get_match_keywordRank(self, keyword, start_date, end_date, day=1):
+    def get_match_keywordRank(self, keyword, start_date=today_date, end_date=today_date-one_day, day=1):
         """
             * 匹配App下某个关键词是否已掉词：
         """
@@ -907,7 +978,7 @@ class Get_App_Keyword(Qimai_Diy_Var):
         else:
             return '掉词'
 
-    def get_match_keywordLost(self, keyword, end_date, start_date, day=0):
+    def get_match_keywordLost(self, keyword, start_date=today_date, end_date=today_date-one_day, day=0):
         """
             * 匹配App下某个关键词是否未覆盖；
         """
@@ -965,57 +1036,52 @@ class Get_App_Comment(Qimai_Diy_Var):
         * 获取App的评论相关数据；
         * 举例①：获取App每天的新增评论数；
         * 举例②：获取App指定星级每天评论数据；
-        typec：评论计量时间单位：默认按天\n
-        star：评论星级，默认5星\n
-        delete：每日评论统计类型，默认全部评论，其他可选例如未删除评论、已删除评论\n
-        orderType：评论排序类型，默认最新评价，其他可选例如最优帮助、最高评价、最低评价\n
-        commentType：评论类型，默认全部评论，其他可选例如开发者回复、未删除、已删除\n
     """
-    def __init__(self, appid, start_date, end_date, typec='day', star='five', delete=-1, orderType='time', commentType='default'):
+    def __init__(self, appid, start_date, end_date):
         Qimai_Diy_Var.__init__(self)
         self.appid = appid
         self.start_date = start_date
         self.end_date = end_date
-        self.typec = typec
-        self.star = star
-        self.delete = delete
-        self.orderType = orderType
-        self.commentType = commentType
 
-    def get_commentRateNum(self):
+    def get_commentRateNum(self, typec='day'):
         """
             * 获取1-5星每个星级每天的评论增删数量；
+            :param typec: 评论计量时间单位：默认按天
         """
-        url = 'https://api.qimai.cn/app/commentRateNum?appid=%s&country=%s&sdate=%s&edate=%s&typec=%s' %(self.appid, self.country, self.start_date, self.end_date, self.typec)
+        url = 'https://api.qimai.cn/app/commentRateNum?appid=%s&country=%s&sdate=%s&edate=%s&typec=%s' %(self.appid, self.country, self.start_date, self.end_date, typec)
         res = session.get(url, headers=headers)
         self.app_commentRateNum = res.json()
         return self.app_commentRateNum
 
-    def get_commentNum(self):
+    def get_commentNum(self, delete=-1):
         """
             * 获取每日已删除未删除评论数量；
+            :param delete: 每日评论统计类型，默认全部评论，其他可选例如未删除评论、已删除评论
         """
-        url = 'https://api.qimai.cn/app/commentNum?appid=%s&country=%s&delete=%s&sdate=%s&edate=%s' %(self.appid, self.country, self.delete, self.start_date, self.end_date)
+        url = 'https://api.qimai.cn/app/commentNum?appid=%s&country=%s&delete=%s&sdate=%s&edate=%s' %(self.appid, self.country, delete, self.start_date, self.end_date)
         res = session.get(url, headers=headers)
         self.app_commentNum = res.json()
         return self.app_commentNum
 
-    def get_comment(self, search_word=''):
+    def get_comment(self, search_word='', star='five', orderType='time', commentType='default'):
         """
             * 获取详细每条评论内容：
-            * 包含星级、用户昵称、评论内容等；
+            * 包含星级、用户昵称、评论内容等(默认5星)；
+            :param star: 评论星级，默认5星
+            :param orderType: 评论排序类型，默认最新评价，其他可选例如最优帮助、最高评价、最低评价
+            :param commentType: 评论类型，默认全部评论，其他可选例如开发者回复、未删除、已删除
         """
-        url = 'https://api.qimai.cn/app/comment?appid=%s&country=%s&sword=&sdate=%s+00:00:00&edate=%s+23:59:59&orderType=%s&commentType=%s&sword=%s' %(self.appid, self.country, self.start_date, self.end_date, self.orderType, self.commentType, search_word)
+        url = 'https://api.qimai.cn/app/comment?appid=%s&country=%s&sword=&sdate=%s+00:00:00&edate=%s+23:59:59&orderType=%s&commentType=%s&sword=%s&star=%s' %(self.appid, self.country, self.start_date, self.end_date, orderType, commentType, search_word, star)
         res = session.get(url, headers=headers)
         self.app_comment = res.json()
         return self.app_comment
 
-    def get_all_commentRateNum(self):
+    def get_all_commentRateNum(self, typec='day'):
         """
             * 获取所有1-5星评论数量汇总为1列：
             * 返回dataframe格式数据；
         """
-        self.get_commentRateNum()
+        self.get_commentRateNum(typec)
         df = pd.DataFrame({})
         for comment_info in self.app_commentRateNum['rateInfo']:
             df_new = Qimai_Outside_Tool(comment_info['data']).list_to_df()
@@ -1025,12 +1091,12 @@ class Get_App_Comment(Qimai_Diy_Var):
         df = df.groupby('日期').sum()
         return df
 
-    def get_Star_commentRateNum(self, star_value='五星'):
+    def get_Star_commentRateNum(self, star_value='五星', typec='day'):
         """
             * 获取指定星级的评论数量汇总：
             * 默认获取5星的，返回dataframe格式数据；
         """
-        self.get_commentRateNum()
+        self.get_commentRateNum(typec)
         df = pd.DataFrame({})
         for comment_info in self.app_commentRateNum['rateInfo']:
             if comment_info['name'] == star_value:
@@ -1164,20 +1230,23 @@ class Get_PreOrder_AppList(Qimai_Diy_Var):
         * 获取预订App列表相关数据；
         * 举例①：获取当前预订列表所有预订产品相关数据；
     """
-    def __init__(self, preOrder_order=1):
+    def __init__(self, preOrder_order=1, price_status=3):
         Qimai_Diy_Var.__init__(self)
         self.preOrder_order = preOrder_order
+        self.price_status = price_status
 
     def get_preOrder_applist(self):
         """
             * 获取时间段内预订App列表产品相关信息；
             * 从类中指定开始结束时间；
             * 返回列表格式数据，列表中都是json；
+            preOrder_order - 预订排序规则，默认按照发布日期；\n
+            price_status - 预订产品价格筛选，默认全部，1为免费，0为付费，3为全部
         """
         self.preOrder_applist_list = []
         page_num = 1
         while True:
-            url = 'https://api.qimai.cn/rank/preOrder?genre=%s&country=%s&order=%s&page=%s' %(self.genre_type, self.country, self.preOrder_order, page_num)
+            url = 'https://api.qimai.cn/rank/preOrder?genre=%s&country=%s&order=%s&status=%s&page=%s' %(self.genre_type, self.country, self.preOrder_order, self.price_status, page_num)
             res = session.get(url, headers=headers)
             self.preOrder_applist_list.append(res.json())
             page_num += 1
@@ -1346,7 +1415,7 @@ class Get_Keyword_HintsRank(Qimai_Diy_Var):
         minPopular: 最小流行度\n
         maxPopular: 最大流行度
     """
-    def __init__(self, change_inc=0, minResult='', maxResult='', minHints='', maxHints='', minPopular='', maxPopular=''):
+    def __init__(self, change_inc=0, run_time=today_date, minResult='', maxResult='', minHints='', maxHints='', minPopular='', maxPopular=''):
         Qimai_Diy_Var.__init__(self)
         self.change_inc = change_inc
         self.minResult = minResult
@@ -1355,6 +1424,7 @@ class Get_Keyword_HintsRank(Qimai_Diy_Var):
         self.maxHints = maxHints
         self.minPopular = minPopular
         self.maxPopular = maxPopular
+        self.run_time = run_time
 
     def get_hints_rank(self):
         """
@@ -1536,9 +1606,10 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
         * 获取免费、付费、畅销榜产品列表；
         * 举例①：获取游戏畅销榜前100产品信息；
     """
-    def __init__(self, snapshot=''):
+    def __init__(self, snapshot='', run_time=today_date):
         Qimai_Diy_Var.__init__(self)
         self.snapshot = snapshot
+        self.run_time = run_time
 
     def get_updateTime_data(self):
         """
@@ -1551,7 +1622,7 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
 
     def get_freeRank_list(self, max_index=1000000):
         """
-            * 获取免费榜产品列表-所有页面：
+            * 获取免费榜产品列表-所有页面，一页50个：
             :param max_index: 获取产品数量
         """
         self.app_freeRank_list = []
@@ -1633,7 +1704,7 @@ class Get_AppDownRevenue_Data(Qimai_Diy_Var):
         * 获取产品预估下载量及预估收入数据；
         * 举例①：获取产品在某时间段内每日预估下载量数据；
     """
-    def __init__(self, appid, start_date, end_date):
+    def __init__(self, appid, start_date=today_date-datetime.timedelta(days=7), end_date=today_date-one_day):
         Qimai_Diy_Var.__init__(self)
         self.appid = appid
         self.start_date = start_date
